@@ -21,12 +21,7 @@
  */
 
 #include "gssapiP_krb5.h"
-
-#if TARGET_OS_MAC
-#include <Kerberos/com_err.h>
-#else
 #include "com_err.h"
-#endif
 
 /* XXXX internationalization!! */
 
@@ -54,7 +49,6 @@ krb5_gss_display_status(minor_status, status_value, status_type,
       return(GSS_S_FAILURE);
 
    if ((mech_type != GSS_C_NULL_OID) &&
-       !g_OID_equal(gss_mech_krb5_v2, mech_type) &&
        !g_OID_equal(gss_mech_krb5, mech_type) &&
        !g_OID_equal(gss_mech_krb5_old, mech_type)) {
        *minor_status = 0;
