@@ -16,7 +16,10 @@
  * this permission notice appear in supporting documentation, and that
  * the name of M.I.T. not be used in advertising or publicity pertaining
  * to distribution of the software without specific, written prior
- * permission.  M.I.T. makes no representations about the suitability of
+ * permission.  Furthermore if you modify this software you must label
+ * your software as modified software and not distribute it in such a
+ * fashion that it might be confused with the original M.I.T. software.
+ * M.I.T. makes no representations about the suitability of
  * this software for any purpose.  It is provided "as is" without express
  * or implied warranty.
  *
@@ -25,6 +28,7 @@
 /*
  * ser_sctx.c - Handle [de]serialization of GSSAPI security context.
  */
+#include "k5-int.h"
 #include "gssapiP_krb5.h"
 
 /*
@@ -39,192 +43,6 @@
  * still be done. --marc
  */
 
-/*
- * Determine the size required for this krb5_gss_enc_desc.
- */
-static krb5_error_code
-kg_enc_desc_size(kcontext, arg, sizep)
-    krb5_context	kcontext;
-    krb5_pointer	arg;
-    size_t		*sizep;
-{
-    krb5_error_code	kret;
-    krb5_gss_enc_desc	*edescp;
-    size_t		required;
-
-    /*
-     * krb5_gss_cred_id_t requires:
-     *	krb5_int32	for KG_ENC_DESC
-     *	krb5_int32	for processed.
-     *	krb5_int32	for trailer.
-     */
-    kret = EINVAL;
-    if ((edescp = (krb5_gss_enc_desc *) arg)) {
-	required = 3*sizeof(krb5_int32);
-	if (edescp->key)
-	    kret = krb5_size_opaque(kcontext,
-				    KV5M_KEYBLOCK,
-				    (krb5_pointer) edescp->key,
-				    &required);
-	else
-	    kret = 0;
-	
-	 /*
-	  * We need to use size_opaque here because we're not sure as to the
-	  * ancestry of this eblock, and we can't be sure that the magic number
-	  * is set in it, so we ASSuME that it's ok.
-	  */
-	if (!kret)
-	    kret = krb5_size_opaque(kcontext,
-				    KV5M_ENCRYPT_BLOCK,
-				    (krb5_pointer) &edescp->eblock,
-				    &required);
-
-	if (!kret)
-	    *sizep += required;
-    }
-    return(kret);
-}
-
-/*
- * Externalize this krb5_gss_enc_desc.
- */
-static krb5_error_code
-kg_enc_desc_externalize(kcontext, arg, buffer, lenremain)
-    krb5_context	kcontext;
-    krb5_pointer	arg;
-    krb5_octet		**buffer;
-    size_t		*lenremain;
-{
-    krb5_error_code	kret;
-    krb5_gss_enc_desc	*enc_desc;
-    size_t		required;
-    krb5_octet		*bp;
-    size_t		remain;
-
-    required = 0;
-    bp = *buffer;
-    remain = *lenremain;
-    kret = EINVAL;
-    if ((enc_desc = (krb5_gss_enc_desc *) arg)) {
-	kret = ENOMEM;
-	if (!kg_enc_desc_size(kcontext, arg, &required) &&
-	    (required <= remain)) {
-	    /* Our identifier */
-	    (void) krb5_ser_pack_int32(KG_ENC_DESC, &bp, &remain);
-
-	    /* Now static data */
-	    (void) krb5_ser_pack_int32((krb5_int32) enc_desc->processed,
-				       &bp, &remain);
-
-	    /* Now pack up dynamic data */
-	    if (enc_desc->key)
-		kret = krb5_externalize_opaque(kcontext,
-					       KV5M_KEYBLOCK,
-					       (krb5_pointer) enc_desc->key,
-					       &bp, &remain);
-	    else
-		kret = 0;
-
-	    if (!kret)
-		kret = krb5_externalize_opaque(kcontext,
-					       KV5M_ENCRYPT_BLOCK,
-					       (krb5_pointer)&enc_desc->eblock,
-					       &bp, &remain);
-	    if (!kret) {
-		(void) krb5_ser_pack_int32(KG_ENC_DESC, &bp, &remain);
-		*buffer = bp;
-		*lenremain = remain;
-	    }
-	}
-    }
-    return(kret);
-}
-
-/*
- * Internalize this krb5_gss_enc_desc.
- */
-static krb5_error_code
-kg_enc_desc_internalize(kcontext, argp, buffer, lenremain)
-    krb5_context	kcontext;
-    krb5_pointer	*argp;
-    krb5_octet		**buffer;
-    size_t		*lenremain;
-{
-    krb5_error_code	kret;
-    krb5_gss_enc_desc	*edescp;
-    krb5_int32		ibuf;
-    krb5_octet		*bp;
-    krb5_encrypt_block	*eblockp;
-    size_t		remain;
-
-    bp = *buffer;
-    remain = *lenremain;
-    kret = EINVAL;
-    /* Read our magic number */
-    if (krb5_ser_unpack_int32(&ibuf, &bp, &remain))
-	ibuf = 0;
-    if (ibuf == KG_ENC_DESC) {
-	kret = ENOMEM;
-
-	/* Get an enc_desc */
-	if ((remain >= (2*sizeof(krb5_int32))) &&
-	    (edescp = (krb5_gss_enc_desc *)
-	     xmalloc(sizeof(krb5_gss_enc_desc)))) {
-	    memset(edescp, 0, sizeof(krb5_gss_enc_desc));
-
-	    /* Get the static data */
-	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
-	    edescp->processed = (int) ibuf;
-
-	    /* edescp->key */
-	    if ((kret = krb5_internalize_opaque(kcontext,
-						KV5M_KEYBLOCK,
-						(krb5_pointer *) &edescp->key,
-						&bp, &remain))) {
-		if (kret == EINVAL)
-		    kret = 0;
-	    }
-
-	    /* edescp->eblock */
-	    if (!kret &&
-		(kret = krb5_internalize_opaque(kcontext,
-						KV5M_ENCRYPT_BLOCK,
-						(krb5_pointer *) &eblockp,
-						&bp, &remain))) {
-		if (kret == EINVAL)
-		    kret = 0;
-	    }
-	    else {
-		/* Successful, copy in allocated eblock to our structure */
-		memcpy(&edescp->eblock, eblockp, sizeof(edescp->eblock));
-		krb5_xfree(eblockp);
-	    }
-
-	    /* trailer */
-	    if (!kret &&
-		!(kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)) &&
-		(ibuf == KG_ENC_DESC)) {
-		*buffer = bp;
-		*lenremain = remain;
-		*argp = (krb5_pointer) edescp;
-	    }
-	    else {
-		if (!kret && (ibuf != KG_ENC_DESC))
-		    kret = EINVAL;
-		if (edescp->eblock.key)
-		    krb5_free_keyblock(kcontext, edescp->eblock.key);
-		if (edescp->eblock.priv && edescp->eblock.priv_size)
-		    krb5_xfree(edescp->eblock.priv);
-		if (edescp->key)
-		    krb5_free_keyblock(kcontext, edescp->key);
-		xfree(edescp);
-	    }
-	}
-    }
-    return(kret);
-}
-
 static krb5_error_code
 kg_oid_externalize(kcontext, arg, buffer, lenremain)
     krb5_context	kcontext;
@@ -233,14 +51,21 @@ kg_oid_externalize(kcontext, arg, buffer, lenremain)
     size_t		*lenremain;
 {
      gss_OID oid = (gss_OID) arg;
+     krb5_error_code err;
      
-     (void) krb5_ser_pack_int32(KV5M_GSS_OID, buffer, lenremain);
-     (void) krb5_ser_pack_int32((krb5_int32) oid->length,
-				buffer, lenremain);
-     (void) krb5_ser_pack_bytes((krb5_octet *) oid->elements,
-				oid->length, buffer, lenremain);
-     (void) krb5_ser_pack_int32(KV5M_GSS_OID, buffer, lenremain);
-     return 0;
+     err = krb5_ser_pack_int32(KV5M_GSS_OID, buffer, lenremain);
+     if (err)
+	 return err;
+     err = krb5_ser_pack_int32((krb5_int32) oid->length,
+			       buffer, lenremain);
+     if (err)
+	 return err;
+     err = krb5_ser_pack_bytes((krb5_octet *) oid->elements,
+			       oid->length, buffer, lenremain);
+     if (err)
+	 return err;
+     err = krb5_ser_pack_int32(KV5M_GSS_OID, buffer, lenremain);
+     return err;
 }
 
 static krb5_error_code
@@ -250,7 +75,6 @@ kg_oid_internalize(kcontext, argp, buffer, lenremain)
     krb5_octet		**buffer;
     size_t		*lenremain;
 {
-     krb5_error_code	kret;
      gss_OID oid;
      krb5_int32 ibuf;
      krb5_octet		*bp;
@@ -260,7 +84,7 @@ kg_oid_internalize(kcontext, argp, buffer, lenremain)
      remain = *lenremain;
 
      /* Read in and check our magic number */
-     if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
+     if (krb5_ser_unpack_int32(&ibuf, &bp, &remain))
 	return (EINVAL);
 
      if (ibuf != KV5M_GSS_OID)
@@ -269,22 +93,35 @@ kg_oid_internalize(kcontext, argp, buffer, lenremain)
      oid = (gss_OID) malloc(sizeof(gss_OID_desc));
      if (oid == NULL)
 	  return ENOMEM;
-     (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+     if (krb5_ser_unpack_int32(&ibuf, &bp, &remain)) {
+	 free(oid);
+	 return EINVAL;
+     }
      oid->length = ibuf;
      oid->elements = malloc(ibuf);
      if (oid->elements == 0) {
 	     free(oid);
 	     return ENOMEM;
      }
-     (void) krb5_ser_unpack_bytes((krb5_octet *) oid->elements,
-				  oid->length, &bp, &remain);
+     if (krb5_ser_unpack_bytes((krb5_octet *) oid->elements,
+			       oid->length, &bp, &remain)) {
+	 free(oid->elements);
+	 free(oid);
+	 return EINVAL;
+     }
      
      /* Read in and check our trailing magic number */
-     if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
-	return (EINVAL);
-
-     if (ibuf != KV5M_GSS_OID)
+     if (krb5_ser_unpack_int32(&ibuf, &bp, &remain)) {
+	 free(oid->elements);
+	 free(oid);
 	 return (EINVAL);
+     }
+
+     if (ibuf != KV5M_GSS_OID) {
+	 free(oid->elements);
+	 free(oid);
+	 return (EINVAL);
+     }
 
      *buffer = bp;
      *lenremain = remain;
@@ -292,7 +129,7 @@ kg_oid_internalize(kcontext, argp, buffer, lenremain)
      return 0;
 }
 
-krb5_error_code
+static krb5_error_code
 kg_oid_size(kcontext, arg, sizep)
     krb5_context	kcontext;
     krb5_pointer	arg;
@@ -323,10 +160,13 @@ kg_queue_externalize(kcontext, arg, buffer, lenremain)
     krb5_octet		**buffer;
     size_t		*lenremain;
 {
-     (void) krb5_ser_pack_int32(KV5M_GSS_QUEUE, buffer, lenremain);
-     g_queue_externalize(arg, buffer, lenremain);
-     (void) krb5_ser_pack_int32(KV5M_GSS_QUEUE, buffer, lenremain);
-     return 0;
+    krb5_error_code err;
+    err = krb5_ser_pack_int32(KV5M_GSS_QUEUE, buffer, lenremain);
+    if (err == 0)
+	err = g_queue_externalize(arg, buffer, lenremain);
+    if (err == 0)
+	err = krb5_ser_pack_int32(KV5M_GSS_QUEUE, buffer, lenremain);
+    return err;
 }
 
 static krb5_error_code
@@ -336,37 +176,42 @@ kg_queue_internalize(kcontext, argp, buffer, lenremain)
     krb5_octet		**buffer;
     size_t		*lenremain;
 {
-     krb5_error_code	kret;
-     gss_OID oid;
      krb5_int32 ibuf;
      krb5_octet		*bp;
      size_t		remain;
+     krb5_error_code	err;
 
      bp = *buffer;
      remain = *lenremain;
 
      /* Read in and check our magic number */
-     if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
+     if (krb5_ser_unpack_int32(&ibuf, &bp, &remain))
 	return (EINVAL);
 
      if (ibuf != KV5M_GSS_QUEUE)
 	 return (EINVAL);
 
-     g_queue_internalize(argp, &bp, &remain);
+     err = g_queue_internalize(argp, &bp, &remain);
+     if (err)
+	  return err;
 
      /* Read in and check our trailing magic number */
-     if ((kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)))
-	return (EINVAL);
-
-     if (ibuf != KV5M_GSS_QUEUE)
+     if (krb5_ser_unpack_int32(&ibuf, &bp, &remain)) {
+	 g_order_free(argp);
 	 return (EINVAL);
+     }
+
+     if (ibuf != KV5M_GSS_QUEUE) {
+	 g_order_free(argp);
+	 return (EINVAL);
+     }
 
      *buffer = bp;
      *lenremain = remain;
      return 0;
 }
 
-krb5_error_code
+static krb5_error_code
 kg_queue_size(kcontext, arg, sizep)
     krb5_context	kcontext;
     krb5_pointer	arg;
@@ -403,23 +248,37 @@ kg_ctx_size(kcontext, arg, sizep)
      * krb5_gss_ctx_id_rec requires:
      *	krb5_int32	for KG_CONTEXT
      *	krb5_int32	for initiate.
-     *	krb5_int32	for mutual.
+     *	krb5_int32	for established.
+     *	krb5_int32	for big_endian.
+     *	krb5_int32	for have_acceptor_subkey.
      *	krb5_int32	for seed_init.
+     *	krb5_int32	for gss_flags.
      *	sizeof(seed)	for seed
+     *	...		for here
+     *	...		for there
+     *	...		for subkey
      *  krb5_int32	for signalg.
      *  krb5_int32	for cksum_size.
      *  krb5_int32	for sealalg.
+     *	...		for enc
+     *	...		for seq
      *	krb5_int32	for endtime.
      *	krb5_int32	for flags.
-     *	krb5_int32	for seq_send.
-     *	krb5_int32	for seq_recv.
-     *	krb5_int32	for established.
-     *	krb5_int32	for big_endian.
+     *	krb5_int64	for seq_send.
+     *	krb5_int64	for seq_recv.
+     *	...		for seqstate
+     *	...		for auth_context
+     *	...		for mech_used
+     *	krb5_int32	for proto
+     *	krb5_int32	for cksumtype
+     *	...		for acceptor_subkey
+     *	krb5_int32	for acceptor_key_cksumtype
      *	krb5_int32	for trailer.
      */
     kret = EINVAL;
     if ((ctx = (krb5_gss_ctx_id_rec *) arg)) {
-	required = 14*sizeof(krb5_int32);
+	required = 16*sizeof(krb5_int32);
+	required += 2*sizeof(krb5_int64);
 	required += sizeof(ctx->seed);
 
 	kret = 0;
@@ -441,14 +300,16 @@ kg_ctx_size(kcontext, arg, sizep)
 				    (krb5_pointer) ctx->subkey,
 				    &required);
 
-	if (!kret)
-	    kret = kg_enc_desc_size(kcontext,
-				    (krb5_pointer) &ctx->enc,
+	if (!kret && ctx->enc)
+	    kret = krb5_size_opaque(kcontext,
+				    KV5M_KEYBLOCK,
+				    (krb5_pointer) ctx->enc,
 				    &required);
 
-	if (!kret)
-	    kret = kg_enc_desc_size(kcontext,
-				    (krb5_pointer) &ctx->seq,
+	if (!kret && ctx->seq)
+	    kret = krb5_size_opaque(kcontext,
+				    KV5M_KEYBLOCK,
+				    (krb5_pointer) ctx->seq,
 				    &required);
 
 	if (!kret)
@@ -464,6 +325,11 @@ kg_ctx_size(kcontext, arg, sizep)
 				    KV5M_AUTH_CONTEXT,
 				    (krb5_pointer) ctx->auth_context,
 				    &required);
+	if (!kret && ctx->acceptor_subkey)
+	    kret = krb5_size_opaque(kcontext,
+				    KV5M_KEYBLOCK,
+				    (krb5_pointer) ctx->acceptor_subkey,
+				    &required);
 	if (!kret)
 	    *sizep += required;
     }
@@ -485,6 +351,11 @@ kg_ctx_externalize(kcontext, arg, buffer, lenremain)
     size_t		required;
     krb5_octet		*bp;
     size_t		remain;
+    krb5int_access kaccess;
+
+    kret = krb5int_accessor (&kaccess, KRB5INT_ACCESS_VERSION);
+    if (kret) 
+        return(kret);
 
     required = 0;
     bp = *buffer;
@@ -500,10 +371,16 @@ kg_ctx_externalize(kcontext, arg, buffer, lenremain)
 	    /* Now static data */
 	    (void) krb5_ser_pack_int32((krb5_int32) ctx->initiate,
 				       &bp, &remain);
-	    (void) krb5_ser_pack_int32((krb5_int32) ctx->gss_flags,
+	    (void) krb5_ser_pack_int32((krb5_int32) ctx->established,
+				       &bp, &remain);
+	    (void) krb5_ser_pack_int32((krb5_int32) ctx->big_endian,
+				       &bp, &remain);
+	    (void) krb5_ser_pack_int32((krb5_int32) ctx->have_acceptor_subkey,
 				       &bp, &remain);
 	    (void) krb5_ser_pack_int32((krb5_int32) ctx->seed_init,
 				       &bp, &remain);
+	    (void) krb5_ser_pack_int32((krb5_int32) ctx->gss_flags,
+				       &bp, &remain);
 	    (void) krb5_ser_pack_bytes((krb5_octet *) ctx->seed,
 				       sizeof(ctx->seed),
 				       &bp, &remain);
@@ -517,13 +394,9 @@ kg_ctx_externalize(kcontext, arg, buffer, lenremain)
 				       &bp, &remain);
 	    (void) krb5_ser_pack_int32((krb5_int32) ctx->krb_flags,
 				       &bp, &remain);
-	    (void) krb5_ser_pack_int32((krb5_int32) ctx->seq_send,
+	    (void) (*kaccess.krb5_ser_pack_int64)((krb5_int64) ctx->seq_send,
 				       &bp, &remain);
-	    (void) krb5_ser_pack_int32((krb5_int32) ctx->seq_recv,
-				       &bp, &remain);
-	    (void) krb5_ser_pack_int32((krb5_int32) ctx->established,
-				       &bp, &remain);
-	    (void) krb5_ser_pack_int32((krb5_int32) ctx->big_endian,
+	    (void) (*kaccess.krb5_ser_pack_int64)((krb5_int64) ctx->seq_recv,
 				       &bp, &remain);
 
 	    /* Now dynamic data */
@@ -551,14 +424,16 @@ kg_ctx_externalize(kcontext, arg, buffer, lenremain)
 					       (krb5_pointer) ctx->subkey,
 					       &bp, &remain);
 
-	    if (!kret)
-		kret = kg_enc_desc_externalize(kcontext,
-					       (krb5_pointer) &ctx->enc,
+	    if (!kret && ctx->enc)
+		kret = krb5_externalize_opaque(kcontext,
+					       KV5M_KEYBLOCK,
+					       (krb5_pointer) ctx->enc,
 					       &bp, &remain);
 
-	    if (!kret)
-		kret = kg_enc_desc_externalize(kcontext,
-					       (krb5_pointer) &ctx->seq,
+	    if (!kret && ctx->seq)
+		kret = krb5_externalize_opaque(kcontext,
+					       KV5M_KEYBLOCK,
+					       (krb5_pointer) ctx->seq,
 					       &bp, &remain);
 
 	    if (!kret && ctx->seqstate)
@@ -570,9 +445,26 @@ kg_ctx_externalize(kcontext, arg, buffer, lenremain)
 					       KV5M_AUTH_CONTEXT,
 					       (krb5_pointer) ctx->auth_context,
 					       &bp, &remain);
-	    
+
+	    if (!kret)
+		kret = krb5_ser_pack_int32((krb5_int32) ctx->proto,
+					   &bp, &remain);
+	    if (!kret)
+		kret = krb5_ser_pack_int32((krb5_int32) ctx->cksumtype,
+					   &bp, &remain);
+	    if (!kret && ctx->acceptor_subkey)
+		kret = krb5_externalize_opaque(kcontext,
+					       KV5M_KEYBLOCK,
+					       (krb5_pointer) ctx->acceptor_subkey,
+					       &bp, &remain);
+	    if (!kret)
+		kret = krb5_ser_pack_int32((krb5_int32) ctx->acceptor_subkey_cksumtype,
+					   &bp, &remain);
+
+	    /* trailer */
+	    if (!kret)
+		kret = krb5_ser_pack_int32(KG_CONTEXT, &bp, &remain);
 	    if (!kret) {
-		(void) krb5_ser_pack_int32(KG_CONTEXT, &bp, &remain);
 		*buffer = bp;
 		*lenremain = remain;
 	    }
@@ -596,7 +488,11 @@ kg_ctx_internalize(kcontext, argp, buffer, lenremain)
     krb5_int32		ibuf;
     krb5_octet		*bp;
     size_t		remain;
-    krb5_gss_enc_desc	*edp;
+    krb5int_access kaccess;
+
+    kret = krb5int_accessor (&kaccess, KRB5INT_ACCESS_VERSION);
+    if (kret)
+        return(kret);
 
     bp = *buffer;
     remain = *lenremain;
@@ -608,7 +504,9 @@ kg_ctx_internalize(kcontext, argp, buffer, lenremain)
 	kret = ENOMEM;
 
 	/* Get a context */
-	if ((remain >= ((10*sizeof(krb5_int32))+sizeof(ctx->seed))) &&
+	if ((remain >= (16*sizeof(krb5_int32)
+			+ 2*sizeof(krb5_int64)
+			+ sizeof(ctx->seed))) &&
 	    (ctx = (krb5_gss_ctx_id_rec *)
 	     xmalloc(sizeof(krb5_gss_ctx_id_rec)))) {
 	    memset(ctx, 0, sizeof(krb5_gss_ctx_id_rec));
@@ -617,9 +515,15 @@ kg_ctx_internalize(kcontext, argp, buffer, lenremain)
 	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
 	    ctx->initiate = (int) ibuf;
 	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
-	    ctx->gss_flags = (int) ibuf;
+	    ctx->established = (int) ibuf;
+	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    ctx->big_endian = (int) ibuf;
+	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    ctx->have_acceptor_subkey = (int) ibuf;
 	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
 	    ctx->seed_init = (int) ibuf;
+	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    ctx->gss_flags = (int) ibuf;
 	    (void) krb5_ser_unpack_bytes((krb5_octet *) ctx->seed,
 					 sizeof(ctx->seed),
 					 &bp, &remain);
@@ -633,12 +537,12 @@ kg_ctx_internalize(kcontext, argp, buffer, lenremain)
 	    ctx->endtime = (krb5_timestamp) ibuf;
 	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
 	    ctx->krb_flags = (krb5_flags) ibuf;
-	    (void) krb5_ser_unpack_int32(&ctx->seq_send, &bp, &remain);
-	    (void) krb5_ser_unpack_int32(&ctx->seq_recv, &bp, &remain);
-	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
-	    ctx->established = (int) ibuf;
-	    (void) krb5_ser_unpack_int32(&ibuf, &bp, &remain);
-	    ctx->big_endian = (int) ibuf;
+	    (void) (*kaccess.krb5_ser_unpack_int64)(&ctx->seq_send, &bp, &remain);
+	    kret = (*kaccess.krb5_ser_unpack_int64)(&ctx->seq_recv, &bp, &remain);
+	    if (kret) {
+		free(ctx);
+		return kret;
+	    }
 
 	    if ((kret = kg_oid_internalize(kcontext, &ctx->mech_used, &bp,
 					   &remain))) {
@@ -669,29 +573,21 @@ kg_ctx_internalize(kcontext, argp, buffer, lenremain)
 		if (kret == EINVAL)
 		    kret = 0;
 	    }
-	    if (!kret) {
-		if ((kret = kg_enc_desc_internalize(kcontext,
-						    (krb5_pointer *) &edp,
-						    &bp, &remain))) {
-		    if (kret == EINVAL)
-			kret = 0;
-		}
-		else {
-		    memcpy(&ctx->enc, edp, sizeof(ctx->enc));
-		    xfree(edp);
-		}
+	    if (!kret &&
+		(kret = krb5_internalize_opaque(kcontext,
+						KV5M_KEYBLOCK,
+						(krb5_pointer *) &ctx->enc,
+						&bp, &remain))) {
+		if (kret == EINVAL)
+		    kret = 0;
 	    }
-	    if (!kret) {
-		if ((kret = kg_enc_desc_internalize(kcontext,
-						    (krb5_pointer *) &edp,
-						    &bp, &remain))) {
-		    if (kret == EINVAL)
-			kret = 0;
-		}
-		else {
-		    memcpy(&ctx->seq, edp, sizeof(ctx->seq));
-		    xfree(edp);
-		}
+	    if (!kret &&
+		(kret = krb5_internalize_opaque(kcontext,
+						KV5M_KEYBLOCK,
+						(krb5_pointer *) &ctx->seq,
+						&bp, &remain))) {
+		if (kret == EINVAL)
+		    kret = 0;
 	    }
 
 	    if (!kret) {
@@ -706,30 +602,40 @@ kg_ctx_internalize(kcontext, argp, buffer, lenremain)
 					       KV5M_AUTH_CONTEXT,
 				       (krb5_pointer *) &ctx->auth_context,
 					       &bp, &remain);
-		
-	    /* Get trailer */
+
+	    if (!kret)
+		kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    ctx->proto = ibuf;
+	    if (!kret)
+		kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    ctx->cksumtype = ibuf;
 	    if (!kret &&
-		!(kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain)) &&
-		(ibuf == KG_CONTEXT)) {
+		(kret = krb5_internalize_opaque(kcontext,
+						KV5M_KEYBLOCK,
+						(krb5_pointer *) &ctx->acceptor_subkey,
+						&bp, &remain))) {
+		if (kret == EINVAL)
+		    kret = 0;
+	    }
+	    if (!kret)
+		kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    ctx->acceptor_subkey_cksumtype = ibuf;
+
+	    /* Get trailer */
+	    if (!kret)
+		kret = krb5_ser_unpack_int32(&ibuf, &bp, &remain);
+	    if (!kret && ibuf != KG_CONTEXT)
+		kret = EINVAL;
+
+	    if (!kret) {
 		*buffer = bp;
 		*lenremain = remain;
 		*argp = (krb5_pointer) ctx;
-	    }
-	    else {
-		if (!kret && (ibuf != KG_CONTEXT))
-		    kret = EINVAL;
-		if (ctx->seq.eblock.key)
-		    krb5_free_keyblock(kcontext, ctx->seq.eblock.key);
-		if (ctx->seq.eblock.priv && ctx->seq.eblock.priv_size)
-		    krb5_xfree(ctx->seq.eblock.priv);
-		if (ctx->seq.key)
-		    krb5_free_keyblock(kcontext, ctx->seq.key);
-		if (ctx->enc.eblock.key)
-		    krb5_free_keyblock(kcontext, ctx->enc.eblock.key);
-		if (ctx->enc.eblock.priv && ctx->enc.eblock.priv_size)
-		    krb5_xfree(ctx->enc.eblock.priv);
-		if (ctx->enc.key)
-		    krb5_free_keyblock(kcontext, ctx->enc.key);
+	    } else {
+		if (ctx->seq)
+		    krb5_free_keyblock(kcontext, ctx->seq);
+		if (ctx->enc)
+		    krb5_free_keyblock(kcontext, ctx->enc);
 		if (ctx->subkey)
 		    krb5_free_keyblock(kcontext, ctx->subkey);
 		if (ctx->there)
